
import jahuwaldt.swing.MainApp;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.prefs.Preferences;

/**
 * This class serves as a collection of preferences for this program.
 *
 * <p> Modified by: Joseph A. Huwaldt </p>
 *
 * @author Joseph A. Huwaldt Date: October 27, 2013
 * @version January 10, 2015
 */
public class AppPreferences implements jahuwaldt.swing.Preferences {

    //  The preferences for this application are stored as a properties list.
    private final Preferences prefs = Preferences.userNodeForPackage(this.getClass());
    
    //  Use a change-listener system to keep the preferences dialog updated
    private final PropertyChangeSupport propertyChange = new PropertyChangeSupport(this);
    
    //  Preference keys
    private static final String LASTPATH = "lastPath";
    

    /**
     * Construct the preferences object for this application. This constructor will locate
     * the preference file and load in any available preferences for the application.
     *
     * @param app Reference to the main application.
     */
    public AppPreferences(MainApp app) {
        //  Do nothing.
    }

    /**
     * Returns the file options to the parent of the last referenced file. Returns null if
     * no last options could be found.
     * 
     * @return The last path that the user opened or saved a file from/to.
     */
    @Override
    public String getLastPath() {
        return prefs.get(LASTPATH, null);
    }

    /**
     * Set the last file options referenced by the user. This is the options to the last
     * parent of the last referenced file.
     * 
     * @param path The path to be stored.
     */
    @Override
    public void setLastPath(String path) {
        String oldPath = prefs.get(LASTPATH, null);
        if ((path != null && !path.equals(oldPath)) || (path == null && oldPath != null)) {
            prefs.put(LASTPATH, path);
        }
    }

    /**
     * Return the preference with the specified key String.
     *
     * @param key The key String identifying the preference to be retrieved.
     * @return The preference that has the specified key.
     */
    @Override
    public String get(String key) {
        return prefs.get(key, null);
    }

    /**
     * Set the preference with the specified key String.
     *
     * @param key The key String identifying the preference to be set.
     * @param value THe String value to store as the preference.
     */
    @Override
    public void set(String key, String value) {
        prefs.put(key, value);
    }

    public synchronized void addPropertyChangeListener(PropertyChangeListener l) {
        propertyChange.addPropertyChangeListener(l);
    }

    public synchronized void addPropertyChangeListener(String propertyName, PropertyChangeListener l) {
        propertyChange.addPropertyChangeListener(propertyName, l);
    }

    public synchronized void removePropertyChangeListener(PropertyChangeListener l) {
        propertyChange.removePropertyChangeListener(l);
    }

    public synchronized void removePropertyChangeListener(String propertyName, PropertyChangeListener l) {
        propertyChange.removePropertyChangeListener(propertyName, l);
    }

    /**
     * Method that displays a dialog that allows the user to change the application
     * preferences. If the application is running in batch mode, this method does nothing.
     */
    @Override
    public void showPreferenceDialog() {
        //  Do nothing.
    }
}
